/*-----------------------------------------------------------------------------
 *
 * Enhydra InstantDB
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright 1997-2000 Lutris Technologies Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * www.enhydra.org/license/epl.html
 *
 * Software distributed under the License is distributed on an "ASIS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * -----------------------------------------------------------------------------
 * $Id: RowEncrypt.java,v 1.3 2000/08/09 12:51:23 pete Exp $
 * -----------------------------------------------------------------------------
 */

/*
 * Set tab=4 for optimum viewing.
 */
package org.enhydra.instantdb;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;
import org.enhydra.instantdb.db.TableEncrypt;

/**
 * This package provides an example of how to write an encryption
 * module for use with InstantDB.
 * <P>
 * It uses the table name, the row number and a supplied password
 * to create a hash. This is then used to initialise a pseudo random
 * number generator. This generates random numbers which are used to
 * XOR the contents of a row of data.
 * <P>
 * WARNING - This is an example module only. For export control 
 * reasons, it uses a deliberately weak encryption algorithm. 
 * CONFIDENTIAL DATA SHOULD NOT BE ENCRYPTED USING THIS MODULE.
 * <P>
 * Cryptographic software should be used only where permitted by
 * local laws and regulations.
 */
public class RowEncrypt implements TableEncrypt {

	String			password;					// the pasword supplied by the user
	MessageDigest	digest;						// used to mangle the password etc.
	Random			randNumbers;				// generaters pseudo random numbers
	byte[]			rands;						// random bytes generated by above
	
	private static final byte[] padding = {1,2,3,4,5,6,7,8,9,0};

	/**
	 * Stores the password and creates a message digest object.
	 */
	public RowEncrypt (String pwd) throws NoSuchAlgorithmException {
		password = pwd;										// save the supplied password
		digest = MessageDigest.getInstance ("MD5");			// get an MD message digest object

		// NOTE - default crypto provider is specified in lib/security/java.security
		// in JDK 1.1.x directory. By default, Sun is the only crypto provider.
		// We use MD5 instead of the more secure SHA for export reasons.

		randNumbers = new Random ();						// create the random number generator

		// NOTE - We use java.util.Random rather than the stronger
		// java.security.SecureRandom for export reasons.
	} // method RowEncrypt

	/**
	 * Adds the table name, password and row number (1st 3 bytes) to
	 * the digest. 1st 4 bytes of the digest are used to set the pseudo
	 * random number generator. This then provides enough bytes to XOR
	 * the contents of the row.
	 */
	private void randomizeRow (String tableName, byte[] rowBuffer, int rowNumber, int[] colOffsets) {
		digest.update (tableName.getBytes());				// add table name to digest
		digest.update (password.getBytes());				// add the password to the digest
		digest.update ((byte)(rowNumber & 0xFF));			// add row - 1st byte
		digest.update ((byte)((rowNumber & 0xFF00)>>8));	// add row - 2nd byte
		digest.update ((byte)((rowNumber & 0xFF0000)>>16));	// add row - 3rd byte
		byte[] digestResult = digest.digest();				// complete the digest calculation

		// The normal standard for password based encryption would be to take 
		// the digest output and use it as the basis of a triple-DES key. The
		// default crypto provider (Sun) doesn't provide any ciphers, so just use
		// the digest to seed a random number generator instead.

		long seed = digestResult[0] + 						// use 1st four bytes as random seed
					(digestResult[1] << 8) +
					(digestResult[2] << 16) +
					(digestResult[3] << 24);
		randNumbers.setSeed (seed);							// seed the random number generator
		if (rands==null) {									// if first time through
			rands = new byte[256];							// create a big enough array for the rands
		} // if
		randNumbers.nextBytes (rands);						// get the next batch of randoms

		// The first few columns are meta-columns which must not be encrypted.
		// So find the first data column and only encrypt from there.
		int dataStart = 0;									// default to encrypting whole buffer
		if (colOffsets != null) {
			dataStart = colOffsets[FIRST_DATA_COLUMN];		// get offset of first datacolumn
		} // if
		for (int i=dataStart; i<rowBuffer.length; i++) {
			rowBuffer[i] = (byte)(rowBuffer[i] ^ rands[i]);	// XOR row with pseudo random data
		} // for
	} // method randomizeRow

	/**
	 * See TableEncrypt for details.
	 */
	public void Encrypt (String tableName, byte[] rowBuffer, int rowNumber, int[] colOffsets) {
		randomizeRow (tableName, rowBuffer, rowNumber, colOffsets);
	} // method Encrypt
	
	public byte[] BinaryEncrypt (String tableName, String columnName, byte[] binaryData, int rowNumber) {
		// create an array big enough to hold padding plus data
		byte[] retBytes = new byte[binaryData.length+padding.length];
		System.arraycopy (padding, 0, retBytes, 0, padding.length);// copy the padding
		System.arraycopy (binaryData, 0,					// copy the data
			retBytes, padding.length, binaryData.length);
		randomizeRow (tableName, retBytes, rowNumber, null);
		return retBytes;
	}

	/**
	 * See TableEncrypt for details.
	 */
	public void Decrypt (String tableName, byte[] rowBuffer, int rowNumber, int[] colOffsets) {
		randomizeRow (tableName, rowBuffer, rowNumber, colOffsets);
	} // method Encrypt

	public byte[] BinaryDecrypt (String tableName, String columnName, byte[] binaryData, int rowNumber) {
		byte[] retBytes = new byte[binaryData.length-padding.length];
		randomizeRow (tableName, binaryData, rowNumber, null);
		System.arraycopy (binaryData, padding.length,		// copy without the padding
			retBytes, 0, retBytes.length);
		return retBytes;
	}


} // class RowEncrypt 
